#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include "Chi2_openmp.h"

int main(int argc ,char* argv[]) {

	FILE *fp;
	size_t size;
	  
	/* Initialize rows, cols, ncases, ncontrols from the user */
	unsigned int rows=atoi(argv[2]);
	unsigned int cols=atoi(argv[3]);
	int ncases=atoi(argv[4]);
	int ncontrols=atoi(argv[5]);
	int nprocs = atoi(argv[6]);
	printf("Individuals=%d SNPs=%d cases=%d controls=%d\n",rows,cols,ncases,ncontrols);

	/*Host variable declaration */

	//int THREADS = 32;
	float* host_results = (float*) malloc(cols * sizeof(float)); 
	struct timeval starttime, endtime;
	clock_t start, end;
	float seconds;
	unsigned int jobs; 
	unsigned long i;
	unsigned long ulone = 1;
	unsigned long ultwo = 2; 

	/*Kernel variable declaration */
        char *line = NULL; size_t len = 0;
	char *token, *saveptr;

	start = clock();

	/* Validation to check if the data file is readable */
	fp = fopen(argv[1], "r");
	if (fp == NULL) {
    		printf("Cannot Open the File");
		return 0;
	}

	size = (size_t)((size_t)rows * (size_t)cols);
	printf("Size of the data = %lu\n",size);

	fflush(stdout);

	unsigned char *dataT = (unsigned char*)malloc((size_t)size);

	if(dataT == NULL) {
	        printf("ERROR: Memory for data not allocated.\n");
	}

        gettimeofday(&starttime, NULL);

        /* Transfer the SNP Data from the file to CPU Memory */
        i=0;
	while (getline(&line, &len, fp) != -1) {
                token = strtok_r(line, " ", &saveptr);
                while(token != NULL){
                        dataT[i] = *token;
                        i++;
                        token = strtok_r(NULL, " ", &saveptr);
                }
/*
                cur=0; read=-1;
                token = strtok(line, " ");
                while(sscanf(line+cur, "%d%n", &tmp, &read)==1){
                        dataT[i] = (char)(((int)'0')+tmp);
                        cur += read;
                        i++;
                }
*/
  	}
	fclose(fp);
        printf("read data\n");
        fflush(stdout);

        gettimeofday(&endtime, NULL);
        seconds+=((double)endtime.tv_sec+(double)endtime.tv_usec/1000000)-((double)starttime.tv_sec+(double)starttime.tv_usec/1000000);

        printf("time to read data = %f\n", seconds);

//	jobs = cols;
//	BLOCKS = (jobs + THREADS - 1)/THREADS;
        jobs = (unsigned int) ((cols+nprocs-1)/nprocs);

        gettimeofday(&starttime, NULL);

	/*Calling the kernel function */

	printf("jobs=%d\n", jobs);

#pragma omp parallel num_threads(nprocs)
	kernel(rows,cols,ncases,ncontrols,dataT,host_results, jobs);

        gettimeofday(&endtime, NULL); seconds=((double)endtime.tv_sec+(double)endtime.tv_usec/1000000)-((double)starttime.tv_sec+(double)starttime.tv_usec/1000000);
	printf("time for kernel=%f\n", seconds);
		
	for(i = 0; i < cols; i++) {
		printf("%f ", host_results[i]);
	}
	printf("\n");

	end = clock();
	seconds = (float)(end - start) / CLOCKS_PER_SEC;
	printf("Total time = %f\n", seconds);

	return 0;

}
